@@ -1,7 +1,7 @@
 package Web::Query;
 our $AUTHORITY = 'cpan:TOKUHIROM';
 # ABSTRACT: Yet another scraping library like jQuery
-$Web::Query::VERSION = '0.32';
+$Web::Query::VERSION = '0.33';
 use strict;
 use warnings;
 use 5.008001;
@@ -104,7 +104,11 @@ sub new_from_html {
     my ($class, $html,$options) = @_;
     my $tree = $class->_build_tree($options);
     $tree->parse_content($html);
-    my $self = $class->new_from_element([$tree->disembowel],undef,$options);
+    my $self = $class->new_from_element([
+            map {
+                ref $_ ? $_ : bless { _content => $_ }, 'HTML::TreeBuilder::XPath::TextNode'
+            } $tree->disembowel
+    ],undef,$options);
     $self->{need_delete}++;
     return $self;
 }
@@ -157,7 +161,7 @@ sub find {
     my ($self, $selector) = @_;
     
     my $xpath = ref $selector ? $$selector : selector_to_xpath($selector, root => './');
-    my @new = map { $_->findnodes($xpath) } @{$self->{trees}};
+    my @new = map { eval{ $_->findnodes($xpath) } } @{$self->{trees}};
     
     return (ref $self || $self)->new_from_element(\@new, $self);
 }
@@ -179,7 +183,7 @@ sub as_html {
     my $self = shift;
 
     my @html = map {
-        ref $_ ? $_->isa('HTML::TreeBuilder::XPath::TextNode') 
+        ref $_ ? ( $_->isa('HTML::TreeBuilder::XPath::TextNode') || $_->isa('HTML::TreeBuilder::XPath::CommentNode' ) )
                         ? $_->getValue 
                         : $_->as_HTML( q{&<>'"}, $self->{indent}, {} )
                : $_ 
@@ -195,6 +199,7 @@ sub html {
         map {
             $_->delete_content;
             my $tree = $self->_build_tree;
+            
             $tree->parse_content($_[0]);
             $_->push_content($tree->disembowel);
         } @{$self->{trees}};
@@ -206,7 +211,7 @@ sub html {
         push @html, join '', map { 
             ref $_ ? $_->as_HTML( q{&<>'"}, $self->{indent}, {}) 
                    : encode_entities($_)
-        } $t->content_list;
+        } eval { $t->content_list };
     }
     
     return wantarray ? @html : $html[0];
@@ -281,9 +286,12 @@ sub data {
 
 sub tagname {
     my $self = shift;
-    my @retval = map { ref $_ eq 'HTML::TreeBuilder::XPath::TextNode' 
-                            ? '#text' 
-                            : $_->tag(@_) 
+    my @retval = map { $_ eq '~comment' ? '#comment' : $_ } 
+                 map { ref $_ eq 'HTML::TreeBuilder::XPath::TextNode'    ? '#text' 
+                     : ref $_ eq 'HTML::TreeBuilder::XPath::CommentNode' ? '#comment'
+                     : ref $_ ? $_->tag(@_)
+                     : '#text'
+                     ;
                 } @{$self->{trees}};
     return wantarray ? @retval : $retval[0];
 }
@@ -316,20 +324,22 @@ sub map {
 sub filter {
     my $self = shift;
 
+    my @new;
+
     if (ref($_[0]) eq 'CODE') {
         my $code = $_[0];
         my $i = 0; 
-        $self->{trees} = +[grep {
+        @new = grep {
             my $tree = $_;
-            local $_ = (ref $self || $self)->new($tree);
+            local $_ = (ref $self || $self)->new_from_element($tree);
             $code->($i++, $_);
-        } @{$self->{trees}}];
-
-        return $self;
+        } @{$self->{trees}};
+    }
+    else {
+        my $xpath = ref $_[0] ? ${$_[0]} : selector_to_xpath($_[0]);
+        @new = grep { $_->matches($xpath) } @{$self->{trees}};        
     }
 
-    my $xpath = ref $_[0] ? ${$_[0]} : selector_to_xpath($_[0]);
-    my @new = grep { $_->matches($xpath) } @{$self->{trees}};        
     return (ref $self || $self)->new_from_element(\@new, $self);
 }
 
@@ -624,7 +634,7 @@ sub DESTROY {
     return unless $_[0]->{need_delete};
 
     # avoid memory leaks
-    $_->delete for grep { ref $_ } @{$_[0]->{trees}};
+    eval { $_->delete } for @{$_[0]->{trees}};
 }
 
 1;
@@ -641,7 +651,7 @@ Web::Query - Yet another scraping library like jQuery
 
 =head1 VERSION
 
-version 0.32
+version 0.33
 
 =head1 SYNOPSIS
 
