@@ -1,7 +1,7 @@
 package Web::Query::LibXML;
 our $AUTHORITY = 'cpan:TOKUHIROM';
 # ABSTRACT: fast, drop-in replacement for Web::Query
-$Web::Query::LibXML::VERSION = '0.32';
+$Web::Query::LibXML::VERSION = '0.33';
 
 use 5.008005;
 use strict;
@@ -9,6 +9,9 @@ use warnings;
 use parent qw/Web::Query Exporter/;
 use HTML::TreeBuilder::LibXML;
 
+# version required for unique_key
+use XML::LibXML 2.0107;
+
 our @EXPORT = qw/wq/;
 
 sub wq { Web::Query::LibXML->new(@_) }
@@ -65,7 +68,7 @@ Web::Query::LibXML - fast, drop-in replacement for Web::Query
 
 =head1 VERSION
 
-version 0.32
+version 0.33
 
 =head1 SYNOPSIS
 
